@@ -1,10 +1,11 @@
 package Test::Output;
+use vars qw($VERSION);
 
 use warnings;
 use strict;
 
 use Test::Builder;
-use Test::Output::Tie;
+use File::Temp qw(tempfile :seekable);
 use Sub::Exporter -setup => {
   exports => [
     qw(output_is output_isnt output_like output_unlike
@@ -58,13 +59,9 @@ my $Test = Test::Builder->new;
 
 Test::Output - Utilities to test STDOUT and STDERR messages.
 
-=head1 VERSION
-
-Version 0.12
-
 =cut
 
-our $VERSION = '0.12';
+$VERSION = '0.14';
 
 =head1 SYNOPSIS
 
@@ -110,7 +107,8 @@ Originally this module was designed not to have external requirements,
 however, the features provided by L<Sub::Exporter> over what L<Exporter>
 provides is just to great to pass up.
 
-Test::Output ties STDOUT and STDERR using Test::Output::Tie.
+Previous versions tie'd STDOUT and STDERR but this one uses seekable
+temp files.
 
 =cut
 
@@ -816,18 +814,8 @@ stdout_from() executes $coderef and captures STDOUT.
 =cut
 
 sub stdout_from (&) {
-  my $test = shift;
-
-  select( ( select(STDOUT), $| = 1 )[0] );
-  my $out = tie *STDOUT, 'Test::Output::Tie';
-
-  &$test;
-  my $stdout = $out->read;
-
-  undef $out;
-  untie *STDOUT;
-
-  return $stdout;
+	my $test = shift;
+	return getfrom($test, 1, 0);
 }
 
 =head2 stderr_from
@@ -840,21 +828,7 @@ stderr_from() executes $coderef and captures STDERR.
 =cut
 
 sub stderr_from (&) {
-  my $test = shift;
-
-  local $SIG{__WARN__} = sub { print STDERR @_ }
-    if $] < 5.008;
-  
-  select( ( select(STDERR), $| = 1 )[0] );
-  my $err = tie *STDERR, 'Test::Output::Tie';
-
-  &$test;
-  my $stderr = $err->read;
-
-  undef $err;
-  untie *STDERR;
-
-  return $stderr;
+  return getfrom(shift, 0, 1);
 }
 
 =head2 output_from
@@ -867,23 +841,7 @@ output_from() executes $coderef one time capturing both STDOUT and STDERR.
 =cut
 
 sub output_from (&) {
-  my $test = shift;
-
-  select( ( select(STDOUT), $| = 1 )[0] );
-  select( ( select(STDERR), $| = 1 )[0] );
-  my $out = tie *STDOUT, 'Test::Output::Tie';
-  my $err = tie *STDERR, 'Test::Output::Tie';
-
-  &$test;
-  my $stdout = $out->read;
-  my $stderr = $err->read;
-
-  undef $out;
-  undef $err;
-  untie *STDOUT;
-  untie *STDERR;
-
-  return ( $stdout, $stderr );
+  return getfrom(shift, 1, 1);
 }
 
 =head2 combined_from
@@ -892,29 +850,12 @@ sub output_from (&) {
   my $combined = combined_from {...};
 
 combined_from() executes $coderef one time combines STDOUT and STDERR, and
-captures them. combined_from() is equivalent to using 2>&1 in UNIX.
+captures them. combined_from() is equivalent to using 2>&1 in UNIX and Windows.
 
 =cut
 
 sub combined_from (&) {
-  my $test = shift;
-
-  select( ( select(STDOUT), $| = 1 )[0] );
-  select( ( select(STDERR), $| = 1 )[0] );
-
-  open( STDERR, ">&STDOUT" );
-
-  my $out = tie *STDOUT, 'Test::Output::Tie';
-  tie *STDERR, 'Test::Output::Tie', $out;
-
-  &$test;
-  my $combined = $out->read;
-
-  undef $out;
-  untie *STDOUT;
-  untie *STDERR;
-
-  return ($combined);
+  return getfrom(shift, 1, 1);
 }
 
 sub _chkregex {
@@ -936,16 +877,71 @@ sub _chkregex {
   return 1;
 }
 
+sub slurp ($) {
+	my $h = shift;
+	$h->seek(0, 0) || die "can't seek: $!";
+	return join("", <$h>);
+}
+
+sub getfrom ($$$) {
+	my ($test, $capout, $caperr) = @_;
+	my ($oldout, $olderr);
+	my ($tmpout, $tmperr);
+
+	if ($capout) {
+		open $oldout, ">&STDOUT" or die "can't dup STDIN: $!";
+		$tmpout = new File::Temp() || die "couldn't create a temporary file to redirect STDOUT: $!";
+		open STDOUT, ">&", $tmpout or die "can't make STDOUT a copy of temp handle: $!";
+	}
+
+	if ($caperr) {
+		open $olderr, ">&STDERR" or die "can't dup STDERR: $!";
+		if (($capout && wantarray) || !$capout) {
+			$tmperr = new File::Temp() || die "couldn't create a temporary file to redirect STDERR: $!";
+			open STDERR, ">&", $tmperr or die "can't make STDERR a copy of temp handle: $!";
+		} else {
+			open STDERR, ">&STDOUT" or die "can't make STDERR a copy of STDOUT: $!";
+		}
+	}
+
+	&$test;
+
+	if ($tmpout) {
+		if ($tmperr) {
+			my ($out, $err) = (slurp($tmpout), slurp($tmperr));
+			open STDOUT, ">&", $oldout or die "can't restore STDOUT: $!";
+			open STDERR, ">&", $olderr or die "can't restore STDERR: $!";
+			close($tmpout);
+			close($tmperr);
+			File::Temp::cleanup();	# This should work but does not (but it's bugged)
+			return $out, $err;
+		} else {
+			my $out = slurp($tmpout);
+			open STDOUT, ">&", $oldout or die "can't restore STDOUT: $!";
+			close($tmpout);
+			File::Temp::cleanup();	# This should work but does not (but it's bugged)
+			return $out;
+		}
+	} else {
+		my $err = slurp($tmperr);
+		open STDERR, ">&", $olderr or die "can't restore STDERR: $!";
+		close($tmperr);
+		File::Temp::cleanup();	# This should work but does not (but it's bugged)
+		return $err;
+	}
+}
+
 =head1 AUTHOR
 
+Currently maintained by brian d foy, C<bdfoy@cpan.org>.
+
 Shawn Sorichetti, C<< <ssoriche@cpan.org> >>
 
 =head1 SOURCE AVAILABILITY
  
 This module is in Github:
 
-L<<a href="http://github.com/ssoriche/test-output/tree/master">http://github.com/ssoriche/test-output/tree/master</a>>
-
+	http://github.com/briandfoy/test-output/tree/master
 
 =head1 BUGS
 
@@ -967,7 +963,7 @@ Thanks to Michael G Schwern for the solution to combining STDOUT and STDERR.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Shawn Sorichetti, All Rights Reserved.
+Copyright 2005-2008 Shawn Sorichetti, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
